/*
* max7219.c
*
* Created: 22.10.2014 18:57:03
*  Author: sschultewolter
*/


#include "max7219.h"
#include "font.h"

#define bit_read(value, bit) (((value) >> (bit)) & 0x01)
#define bit_set(value, bit) ((value) |= (1UL << (bit)))
#define bit_clear(value, bit) ((value) &= ~(1UL << (bit)))
#define bit_write(value, bit, bitvalue) (bitvalue ? bit_set(value, bit) : bit_clear(value, bit))


/************************************************************************/
/* Core                                                                 */
/************************************************************************/

void max7219_clock_out(uint8_t data)
{
	for(uint8_t i = 0; i < 8; i++)
	{
		// CLK auf LOW setzen
		MAX7219_PORT &= ~(1 << MAX7219_CLK);
		
		if(data & (1 << (7 - i))) MAX7219_PORT |= (1 << MAX7219_DIN);
		else MAX7219_PORT &= ~(1 << MAX7219_DIN);
		
		// CLK auf HIGH setzen, Daten uebernehmen
		MAX7219_PORT |= (1 << MAX7219_CLK);
	}
}

void max7219_send(uint8_t ic, uint8_t reg, uint8_t data)
{
	uint8_t i = 0;
	if(ic < MAX7219_NUM_IC)
	{
		// LOAD auf LOW setzen
		MAX7219_PORT &= ~(1 << MAX7219_LOAD);
		
		for(i = ic; i < (MAX7219_NUM_IC - 1); i++)
		{
			max7219_clock_out(MAX7219_NO_OP);
			max7219_clock_out(MAX7219_NO_OP);
		}
		
		max7219_clock_out(reg);
		max7219_clock_out(data);
		
		for(i = 0; i < ic; i++)
		{
			max7219_clock_out(MAX7219_NO_OP);
			max7219_clock_out(MAX7219_NO_OP);
		}
		
		// LOAD auf HIGH, steigende Flanke uebernimmt Daten in adressiertes Register
		MAX7219_PORT |= (1 << MAX7219_LOAD);
	}
}

void max7219_init(void)
{
	MAX7219_DDR |= (1 << MAX7219_CLK) | (1 << MAX7219_LOAD) | (1 << MAX7219_DIN);

	MAX7219_PORT |= (1 << MAX7219_LOAD);
	MAX7219_PORT &= ~((1<< MAX7219_CLK) | (1 << MAX7219_DIN));
	
	MAX7219_DDR |= (1 << MAX7219_DEBUG) | (1 << MAX7219_DEBUG2);
	MAX7219_PORT |= (1 << MAX7219_DEBUG) | (1 << MAX7219_DEBUG2);
	
	for(uint8_t ic = 0; ic < MAX7219_NUM_IC; ic++)
	{
		max7219_send(ic, MAX7219_DEC_MODE, 0);
		max7219_send(ic, MAX7219_INTENSITY, 0);
		max7219_send(ic, MAX7219_SCAN_LIMIT, 7);
		max7219_send(ic, MAX7219_DISP_TEST, 0);
	}
	
	max7219_clear();
	max7219_on();
}


/************************************************************************/
/* Einstellungen                                                        */
/************************************************************************/

void max7219_intensity(uint8_t intensity)
{
	if(intensity > 15) return;
	
	for(uint8_t ic = 0; ic < MAX7219_NUM_IC; ic++) max7219_send(ic, MAX7219_INTENSITY, intensity);
}

void max7219_off(void)
{
	for(uint8_t ic = 0; ic < MAX7219_NUM_IC; ic++) max7219_send(ic, MAX7219_SHUTDOWN, 0);
}

void max7219_on(void)
{
	for(uint8_t ic = 0; ic < MAX7219_NUM_IC; ic++) max7219_send(ic, MAX7219_SHUTDOWN, 1);
}

void max7219_disp_test(uint8_t disp_test)
{
	if(disp_test > 1) return;
	
	for(uint8_t ic = 0; ic < MAX7219_NUM_IC; ic++) max7219_send(&ic, MAX7219_DISP_TEST, disp_test);
}

void max7219_clear(void)
{
	for(uint8_t ic = 0; ic < MAX7219_NUM_IC; ic++)
	{
		for(uint8_t i = 0; i < 8; i++) max7219_send(ic, i + 1, 0);
	}
}


/************************************************************************/
/* Grafikzwischenspeicher                                               */
/************************************************************************/

uint8_t max7219_buffer[MAX7219_MAX_COL+1];
uint8_t max7219_buffer2[MAX7219_MAX_COL+1];

void max7219_buffer_write(int16_t col, uint8_t value)
{
	if(col > MAX7219_MAX_COL || col < 0) return;
	max7219_buffer[col] = value;
}

void max7219_buffer_clear(void)
{
	for(int col = 0; col <= MAX7219_MAX_COL; col++)
	max7219_write_buffer(col, 0x00);
}

void max7219_buffer_send(void)
{
	uint8_t ic = 0;
	while(ic < MAX7219_NUM_IC)
	{
		uint8_t value = 0;
		for(int8_t i = 0; i <  8; i++)
		{
			for(int8_t t = 0; t < 8; t++) bit_write(value, t, bit_read(max7219_buffer[(ic * 8) + 7 - t], 7 - i));
			max7219_send(ic, (i % 8) + 1, value);
			value = 0;
		}
		ic++;
	}
}


/************************************************************************/
/* Hilfsfunktionen                                                      */
/************************************************************************/

uint16_t max7219_width_string(char *s)
{
	uint8_t width = 0;
	uint8_t i = 0;
	while(i < strlen(s))
	{
		width += pgm_read_byte(&font_bold[s[i]][0]);
		i++;
	}
	return width + (i-1);
}


/************************************************************************/
/* Textausgabe                                                          */
/************************************************************************/

/*void max7219_puts_scroll(char *s)
{
	const uint8_t space = 6;							// Abstand zwischen den Wiederholungen in Spalten
	uint16_t size_str = max7219_width_string(s);	// Benoetigte Spalten fuer einen kompletten String
	
	int16_t start = 0;
	static int16_t x;										// Durchlaufender Versatz

	for(uint8_t c = 0; c < strlen(s); c++)
	{
		// Liest die Breite eines Zeichens aus. Breite fest definiert als 0. Stelle
		for(uint8_t i = 0; i < pgm_read_byte(&font_bold[s[c]][0])+1; i++)
		{
			for(int y = 0; y < MAX7219_MAX_COL / size_str; y++)	// 1)
			{
				max7219_buffer_write((start-x+(y*(size_str + space))), pgm_read_byte(&font_bold[s[c]][i]+1));
				if(i == pgm_read_byte(&font_bold[s[c]][0])) max7219_buffer_write((start-x+(y*(size_str + space))), 0x00);
			}
			start++;
		}
	}

	if(x++ > (size_str+space)-2) x = 0;
	max7219_buffer_send();
}*/

void max7219_puts_scroll(char *s, int8_t *run)
{
MAX7219_PORT |= (1 << MAX7219_DEBUG);	// DEBUG

	int16_t size_str = max7219_width_string(s);
	static int16_t offset = MAX7219_MAX_COL;
	int16_t x = offset;
	
	if(*run == 0) return;
	// Ausfuehren, solange noch mindesten 1 Durchlauf ansteht,
	// oder -1 fuer unendlich Durchlaeufe
	else if(*run < 0 || *run > 0)
	{
		for(uint8_t c = 0; c < strlen(s); c++)
		{
			// Spalten des Zeichens ermitteln (0 - 8 Spalten)
			uint8_t size_char = pgm_read_byte(&font_bold[s[c]][0]);
			
			for(uint8_t i = 0; i < size_char; i++)
			{
				max7219_buffer_write(i+x, pgm_read_byte(&font_bold[s[c]][i+1]));
			}
			
			// Leere Spalte nach jedem Zeichen schreiben
			max7219_buffer_write(x+size_char, 0x00);
			
			x += size_char+1;
		}
		
		// Speicher an MAX7219 senden
		max7219_buffer_send();
		
		// Wenn Ende erreicht, von vorne anfangen
		if(--offset < 0 - size_str ) { offset = MAX7219_MAX_COL; *run -= 1; }
	}
	MAX7219_PORT &= ~(1 << MAX7219_DEBUG);	// DEBUG ENDE
}

void max7219_puts(char *s, uint8_t align)
{
	int16_t start = 0;
	uint16_t size_str = max7219_width_string(s);

	if(align == LEFT) start = 0;
	else if(align == RIGHT)
	{
		if(size_str > MAX7219_MAX_COL+1) return;
		start = MAX7219_MAX_COL - size_str + 1;
	}
	else if(align == CENTER)
	{
		if(size_str > MAX7219_MAX_COL+1) return;
		start = (MAX7219_MAX_COL - size_str + 1) / 2;
	}
	else return;

	for(uint8_t c = 0; c < strlen(s); c++)
	{
		// Liest die Breite eines Zeichens aus. Breite fest definiert als 0. Stelle
		for(uint8_t i = 0; i < pgm_read_byte(&font_bold[s[c]][0])+1; i++)
		{
			max7219_buffer_write(start, pgm_read_byte(&font_bold[s[c]][i]+1));
			if(i == pgm_read_byte(&font_bold[s[c]][0])) max7219_buffer_write(start, 0x00);
			start++;
		}
	}
	max7219_buffer_send();
}

/************************************************************************/
/* Debug (Grafikspeicher auf Console(Putty) wiedergeben                 */
/************************************************************************/

void max7219_buffer_debug(void)
{

	uart_puts("\r\n");
	uart_putc('\n');

	uart_puts(" +");
	for(int k = 0; k < 2*MAX7219_MAX_COL+1; k++)
	{
		if(k%16 == 15) uart_putc('+');
		else if(!(k%2)) uart_putc(' ');
		else uart_putc('-');
	}
	uart_puts("\r\n");
	
	for(int j = 0; j < 8; j++)
	{
		uart_puts(" |");
		for(int i = 0; i < MAX7219_MAX_COL+1; i++)
		{
			uint8_t col = max7219_buffer2[i];
			if(bit_read(col, 7-j)) uart_puts("\033[31mx\033[0m");
			else uart_putc(' ');
			
			if(i%8 == 7 && i < MAX7219_MAX_COL)	uart_putc('|');
			else if(i != MAX7219_MAX_COL) uart_putc(' ');
		}
		uart_putc('|');
		uart_puts("\r\n");
	}
	
	uart_puts(" +");
	for(int k = 0; k < 2*MAX7219_MAX_COL+1; k++)
	{
		if(k%16 == 15) uart_putc('+');
		else if(!(k%2)) uart_putc(' ');
		else uart_putc('-');
	}
	//for(int k = 0; k < MAX7219_MAX_COL+3; k++) uart_puts(" .");
}
