/*
* max7219.h
*
* Created: 22.10.2014 18:56:53
*  Author: sschultewolter
*/


#ifndef MAX7219_H_
#define MAX7219_H_

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <string.h>
#include <util/delay.h>

#define MAX7219_DDR		DDRC
#define MAX7219_PORT	PORTC

#define MAX7219_CLK		PINC3	// MAX7219.CLK
#define MAX7219_LOAD	PINC1	// MAX7219.LOAD
#define MAX7219_DIN		PINC2	// MAX7219.DIN MOSI
#define MAX7219_DEBUG	PINC0
#define MAX7219_DEBUG2	PINC4

#define MAX7219_NUM_IC	4	// Anzahl MAX7219
#define MAX7219_MAX_COL	(MAX7219_NUM_IC * 8) - 1

#define MAX7219_NO_OP		0x00
#define MAX7219_DIGIT_0		0x01
#define MAX7219_DIGIT_1		0x02
#define MAX7219_DIGIT_2		0x03
#define MAX7219_DIGIT_3		0x04
#define MAX7219_DIGIT_4		0x05
#define MAX7219_DIGIT_5		0x06
#define MAX7219_DIGIT_6		0x07
#define MAX7219_DIGIT_7		0x08
#define MAX7219_DEC_MODE	0x09
#define MAX7219_INTENSITY	0x0A
#define MAX7219_SCAN_LIMIT	0x0B
#define MAX7219_SHUTDOWN	0x0C
#define MAX7219_DISP_TEST	0x0F

#define LEFT	0
#define RIGHT	1
#define CENTER	2


/************************************************************************/
/* Core                                                                 */
/************************************************************************/

void max7219_clock_out(uint8_t data);

void max7219_send(uint8_t ic, uint8_t reg, uint8_t data);

void max7219_init(void);


/************************************************************************/
/* Einstellungen                                                        */
/************************************************************************/

void max7219_intensity(uint8_t intensity);

void max7219_off(void);

void max7219_on(void);

void max7219_disp_test(uint8_t disp_test);

void max7219_clear(void);


/************************************************************************/
/* Grafikzwischenspeicher                                               */
/************************************************************************/

void max7219_puts_scroll1(char *s, int8_t *run);

//void max7219_buffer_write(int16_t col, uint8_t value);

void max7219_buffer_clear(void);

void max7219_buffer_send(void);


/************************************************************************/
/* Hilfsfunktionen                                                      */
/************************************************************************/

uint16_t max7219_width_string(char *s);


/************************************************************************/
/* Textausgabe                                                          */
/************************************************************************/

void max7219_puts_scroll(char *s, int8_t *run);

void max7219_puts(char *s, uint8_t align);


#endif /* MAX7219_H_ */